from typing import Any
from typing import Dict


async def get_appended_prefix(
    hub,
    ctx,
    data: dict = None,
) -> Dict[str, Any]:
    if data:
        for k, v in data.items():
            if ("_ref" in k and isinstance(v, str)) and (
                ("name=" not in v) and ("/api" not in v)
            ):
                obj_prefix = k.split("_ref")[0]
                new_value = await hub.tool.nsx_alb.session.append_prefix(
                    ctx, obj_prefix=obj_prefix, value=v
                )
                data.update({k: new_value})
            if "_ref" in k and isinstance(v, list):
                new_value_list = []
                for index in range(len(data[k])):
                    if ("name=" not in data[k][index]) and (
                        "/api" not in data[k][index]
                    ):

                        obj_prefix = k.split("_refs")[0]
                        new_value = await hub.tool.nsx_alb.session.append_prefix(
                            ctx, obj_prefix=obj_prefix, value=data[k][index]
                        )
                        new_value_list.append(new_value)
                data.update({k: new_value_list})
    return data



import copy
from re import sub

from dict_tools.data import NamespaceDict


def snake_case(s):
    return "_".join(
        sub(
            "([A-Z][a-z]+)", r" \1", sub("([A-Z]+)", r" \1", s.replace("-", " "))
        ).split()
    ).lower()


def run(hub, ctx):
    # This customization is used to create an additional folder "alb" to have a 5 segment function reference call
    nsx_resources = [
        "Cloud",
        "VirtualService",
        "Pool",
        "HealthMonitor",
        "VsVip",
        "ApplicationProfile",
        "NetworkProfile",
        "ApplicationPersistenceProfile",
        "SSLProfile",
        "ServiceEngineGroup",
    ]
    updated_cloud_spec: NamespaceDict = copy.deepcopy(ctx.cloud_spec)
    for name, plugin in ctx.cloud_spec.get("plugins").items():
        new_plugin = updated_cloud_spec.get("plugins").pop(name)
        for k in nsx_resources:
            if k.lower() == name:
                new_name = snake_case(k)
                break
        new_resource_name = "alb." + new_name
        for func_name, func_data in new_plugin.get("functions").items():
            func_data.get("hardcoded", {})["resource_name"] = new_resource_name
        updated_cloud_spec["plugins"][new_resource_name] = new_plugin
    ctx.cloud_spec = updated_cloud_spec
